@@ -29,7 +29,7 @@ sys.path.append("/usr/lib/asterisk-phonepatch")
 import templateparser, dtmf, radio
 import radiocontrol, daemonize
 
-__version__ = "$Revision: 1.5 $"
+__version__ = "$Revision: 1.7 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Asterisk', 'Sox', 'Festival', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -91,7 +91,7 @@ class Phonepatch:
 		self.set_state("phonepatch")
 			
 		self.phonepatch_extension = None
-		self.phonepatch_php = None
+		self.phonepatch_phpconfig = None
 
 		# Asterisk definitions
 		self.asterisk_fifo = os.path.join("/tmp/php_fifo.raw")
@@ -119,8 +119,12 @@ class Phonepatch:
 				self.signals[value] = var
 
 		self.background = False
-		self.genctcss = None
+		self.ctcss_tx = None
 		self.interface = None
+		self.send_signal = None
+		self.pidfile_created = False
+
+		self.set_configuration_sections()
 
 	####################################
 	def signal_handler(self, signum, frame):
@@ -138,12 +142,12 @@ class Phonepatch:
 		if signum == signal.SIGUSR1 and not self.sleep_signal:
 			self.debug("signal_handler: phonepatch daemon paused")
 			self.sleep_signal = True
-		elif signum == signal.SIGUSR2 and self.stopped:
+		elif signum == signal.SIGUSR2 and not self.continue_signal:
 			self.debug("signal_handler: phonepatch daemon waked up")
-			self.stopped = False
-		elif signum == signal.SIGALRM and self.stopped:
+			self.continue_signal = True
+		elif signum == signal.SIGALRM and not self.continue_signal:
 			self.debug("signal_handler: phonepatch daemon timed out")
-			self.stopped = False
+			self.continue_signal = True
 		elif signum == signal.SIGHUP:
 			if self.state == "daemon":
 				self.debug("signal_handler: phonepatch daemon killed with SIGHUP")
@@ -158,40 +162,59 @@ class Phonepatch:
 	####################################
 	def init_php(self, phonepatch):	
 		# Initialize DTMF decoder
-		if not phonepatch: self.debug("Phonepatch name not given", 1)
-		if phonepatch not in self.configuration: self.debug("Phonepatch name not found in configuration: %s" %phonepatch, 1)
-		self.phonepatch_php = phonepatch
-		#self.samplerate = self.getconf("soundcard_samplerate")
+		if not phonepatch: 
+			self.debug("init_php: phonepatch name not given")
+			return
+		if phonepatch not in self.configuration: 
+			self.debug("init_php: phonepatch name not found in configuration: %s" %phonepatch)
+			return
+		self.phonepatch_phpconfig = phonepatch
 		self.samplerate = self.asterisk_samplerate
 		self.dtmf_decoder = dtmf.Decoder(samplerate = self.samplerate, \
 			channels = self.asterisk_channels, \
 			sensibility = self.getconf("dtmf_sensibility"), \
 			verbose = False)
 		
-		self.pidfile = os.path.join(PIDFILE_DIR, self.phonepatch_php + ".pid")
+		self.pidfile = os.path.join(PIDFILE_DIR, self.phonepatch_phpconfig + ".pid")
 		self.language = self.getconf("language")
 		self.sounds_dir = self.getconf("sounds_dir")
 		self.outcalls_dir = self.getconf("spool_dir") 
 		self.festival_gain = self.getconf("festival_audio_gain")
+		return phonepatch
 
 	####################################
 	def getconf(self, parameter, phpext=None):
 		if not phpext:
 			phpext = self.phonepatch_extension
-		if phpext and parameter in self.configuration[phpext]:
+		if phpext and phpext in self.phonepatch_extensions and parameter in self.configuration[phpext]:
 			return self.configuration[phpext][parameter]
-		php = self.phonepatch_php
-		if php and parameter in self.configuration[php]:
-			return self.configuration[php][parameter]
-			
-		if self.configuration.has_key(GLOBAL_SECTION) and parameter in self.configuration[GLOBAL_SECTION]:
-			return self.configuration[GLOBAL_SECTION][parameter]
-		if parameter in self.configuration[DEFAULT_SECTION]:
-			value = self.configuration[DEFAULT_SECTION][parameter]
-			self.debug("getconf: parameter not defined, returning default: %s=%s" %(parameter, value))
+		phpconfig = self.phonepatch_phpconfig
+		if phpconfig and phpconfig in self.phonepatch_configs and parameter in self.configuration[phpconfig]:
+			return self.configuration[phpconfig][parameter]
+		if self.phonepatch_global and parameter in self.configuration[self.phonepatch_global]:
+			return self.configuration[self.phonepatch_global][parameter]
+		if parameter in self.configuration[self.phonepatch_default]:
+			value = self.configuration[self.phonepatch_default][parameter]
+			self.debug("getconf: %s not defined, default returned: %s" %(parameter, value))
 			return value
 		self.debug("getconf: unknown parameter: %s" %parameter, 1)
 
+	####################################
+	def set_configuration_sections(self):
+		"""Update configuration sections: phonepatch_conf, phonepatch_extension, phonepatch_global"""
+		self.phonepatch_configs = []
+		self.phonepatch_extensions = []
+		self.phonepatch_global = None
+		self.phonepatch_default = DEFAULT_SECTION
+		for section in self.configuration:
+			if section == GLOBAL_SECTION:
+				self.phonepatch_global = section
+			elif type(section) == str and section.find("phonepatch") == 0:
+				self.phonepatch_configs.append(section)
+			elif section != DEFAULT_SECTION:
+				self.phonepatch_extensions.append(section)
+
+
 	###############################
 	def set_state(self, state):
 		"""Set phonepatch state (incall/outcall/daemon)"""
@@ -200,7 +223,7 @@ class Phonepatch:
 		else: self.state_string = ""
 
 	###############################
-	def debug(self, log, exit = None, delete_pidfile = True):
+	def debug(self, log, exit=None):
 		"""Output debug lines in verbose mode"""
 		if not self.verbose: return
 		if exit: log = "fatal error - " + log
@@ -213,7 +236,8 @@ class Phonepatch:
 		
 		# Exit with code error if "exit" parameter given
 		if exit != None: 
-			if delete_pidfile: self.delete_pidfile()
+			if self.pidfile_created: self.delete_pidfile()
+			if self.send_signal: self.signal_daemon(self.send_signal)
 			sys.exit(exit)
 
 	###############################
@@ -291,8 +315,7 @@ class Phonepatch:
 					"get_carrier_response": self.getconf("command_get_carrier_response"), }
 				self.radio_control = radiocontrol.RadioControl("command", device, command_options=self.command_options)
 
-			else:
-				self.debug("open_radio: syntax error on radio_control: %s" %control, exit = 1)
+			else: self.debug("open_radio: syntax error on radio_control: %s" %control, exit = 1)
 			if self.getconf("ptt"):
 				self.ptt = Container(set=self.radio_control.set_ptt, get=self.radio_control.get_ptt, \
 					threshold = self.getconf("ptt_threshold_signal"), \
@@ -327,7 +350,7 @@ class Phonepatch:
 	def open_interface(self, fifo_file):
 		"""Opens phonepatch interface between Asterisk and radio:
 		
-		- Soundcard -> Phonepath FIFO -> Asterisk (GET DATA command)
+		- Soundcard -> Phonepatch FIFO -> Asterisk (GET DATA command)
 		- Asterisk (file descriptor 3) > Soundcard
 		"""
 		# Way 1: soundcard -> php_fifo -> asterisk
@@ -340,7 +363,7 @@ class Phonepatch:
 		command = 'GET DATA %s ""' %base_file
 		
 		try: self.agi_command(command)
-		except: self.debug("open_interface: error sending AGI command: %s" %command)
+		except: self.debug("open_interface: error sending AGI command: %s" %command); sys.exit(1)
 			
 		# Open fifo (asterisk input) for writing
 		self.asterisk_in = open(fifo_file, "w")
@@ -386,8 +409,12 @@ class Phonepatch:
 		"""
 		audio_buffer = ""
 		# Look for files
-		for afile in [self.language, ""]:
-			cfile = os.path.join(self.sounds_dir, afile, audio_file)
+		if os.path.isabs(audio_file):
+			paths = [audio_file]
+		else:
+			paths = [os.path.join(self.sounds_dir, directory, audio_file) for directory in [self.language, ""]]
+		
+		for cfile in paths:
 			if os.path.isfile(cfile):
 				break
 		else:
@@ -396,7 +423,7 @@ class Phonepatch:
 		
 		# Convert file to raw format with sox, so the soundcard can play it
 		command = "sox %s -t raw -r%d %s -" %(cfile, self.asterisk_samplerate, self.sox_pars)
-		self.debug("play_file: sox spawned: %s" %os.path.join(afile, audio_file))
+		self.debug("play_file: sox spawned: %s" %os.path.join(cfile, audio_file))
 		audio_buffer = self.command_output(command)
 		if not audio_buffer: 
 			self.debug("play_file: sox returned error")
@@ -414,6 +441,7 @@ class Phonepatch:
 		"""
 		# Some sanity checks
 		if not play_radio and not play_asterisk or args == None: return
+		
 		if play_radio: self.debug("play: playing to soundcard: %s" %str(args))
 		if play_asterisk: self.debug("play: playing to asterisk: %s" %str(args))
 		try: last_playargs = self.last_playargs
@@ -428,9 +456,7 @@ class Phonepatch:
 			self.raw_buffer = ""
 			for option in args.split(","):
 				if not option: continue
-				try: option = option.strip().replace("%c", self.agi_keys["callerid"])
-				except: pass
-				try: option = option.strip().replace("%d", self.destination)
+				try: option = option.strip().replace("%u", self.destination)
 				except: pass
 				if option[0] == self.festival_indicator:
 					self.raw_buffer += self.play_text(option[len(self.festival_indicator):])
@@ -442,7 +468,7 @@ class Phonepatch:
 				return 0
 			self.last_playargs = args
 		
-		# If <max_time> defined, calculate maximum amount of bytes to write
+		# If max_time defined, calculate maximum amount of bytes to write
 		buffer = self.raw_buffer
 		written = 0
 					
@@ -469,7 +495,7 @@ class Phonepatch:
 				self.radio.set_ptt(True)
 				pttflag = True
 			if play_radio:
-				try: self.radio.send_audio(buffer[:self.buffer_size], self.genctcss)
+				try: self.radio.send_audio(buffer[:self.buffer_size], self.ctcss_tx)
 				except: self.debug("play: radio send_audio error"); return
 			if play_asterisk:
 				if not self.flush_asterisk(False) and not play_radio: break
@@ -499,12 +525,12 @@ class Phonepatch:
 		return audioop.mul(buffer, self.sample_width, audio_gain)
 
 	#########################################
-	def get_ctcss_freq(self, ctcss_id):
+	def get_ctcss(self, ctcss_id):
 		if not ctcss_id or ctcss_id == "off": return
 		if ctcss_id in PL_CODES: ctcss_freq = PL_CODES[ctcss_id]
 		else: ctcss_freq = ctcss_id
 		try: ctcss_freq = float(ctcss_freq)
-		except: self.debug("get_ctcss_freq: invalid CTCSS frequency: %s" %(str(ctcss_id))); return
+		except: self.debug("get_ctcss: invalid CTCSS frequency: %s" %(str(ctcss_id))); return
 		return ctcss_freq
 
 	###################################
@@ -562,7 +588,7 @@ class Phonepatch:
 				buffer = self.secure_os(os.read, self.asterisk_out, self.buffer_size)
 				if not buffer: self.debug("audio_loop: asterisk closed its read-descriptor"); break_reason = "asterisk"; break
 				buffer = self.set_gain(buffer, self.getconf("radio_audio_gain"))
-				self.radio.vox_toradio(buffer, self.genctcss)
+				self.radio.vox_toradio(buffer, self.ctcss_tx)
 				
 			if self.audio_fd in retsel[0]:
 				# Radio -> Asterisk
@@ -578,11 +604,13 @@ class Phonepatch:
 					if keys: self.debug("audio_loop: DTMF keys received: %s" %("".join(keys)))
 					if self.getconf("hangup_button") in keys:
 						self.debug("audio_loop: hangup DTMF button received")
+						break_reason = "user"
 						break
 			
 			# If time_limit defined, close interface at that time
 			if time_limit and time.time() >= time_limit:
 				self.debug("audio_loop: call time-limit reached: %0.2f seconds" %self.getconf("call_limit"))
+				break_reason = "timeout"
 				break
 		
 		end_audio = self.getconf("end_audio")
@@ -601,46 +629,6 @@ class Phonepatch:
 		self.asterisk_in.close()
 		os.unlink(self.asterisk_fifo)
 		
-	###################################
-	def detect_dtmf(self, dtmf_key, timeout=None, need_ctcss=False):
-		"""Detect dtmf_key in audio from radio link and return it if found"""
-		if not self.dtmf_decoder: return 
-		if timeout: timeout_time = time.time() + timeout
-		self.sleep_signal = False
-		while 1:
-			if self.sleep_signal and self.state == "daemon":
-				self.sleep_daemon(self.getconf("call_limit"))
-			if not self.flush_asterisk(): raise IOError, "Asterisk closed descriptor"
-			buffer = self.radio.read_audio(self.buffer_size)
-			if not buffer: self.debug("detect_dmtf: no buffer from radio"); break
-			if need_ctcss: self.radio.decode_ctcss(buffer)
-			keys = self.dtmf_decoder.decode_buffer(buffer)
-			for key in keys: 
-				self.debug("detect_dtmf: DTMF button received: %s" %key)
-			if dtmf_key in keys:				
-				if need_ctcss and not self.check_ctcss():
-					self.debug("detect_dtmf: CTCSS tone needed but not detected")
-					continue
-				return dtmf_key
-			if timeout and time.time() >= timeout_time:
-				self.debug("detect_dmtf: timeout reached")
-				return
-
-	###################################
-	def detect_ctcss(self, ctcss, timeout=None):
-		"""Wait for CTCSS tone and return if found"""
-		if timeout: timeout_time = time.time() + timeout
-		while 1:
-			if not self.flush_asterisk(): raise IOError, "Asterisk closed descriptor"
-			buffer = self.radio.read_audio(self.buffer_size)
-			if not buffer: break
-			self.radio.decode_ctcss(buffer)
-			tone = self.radio.get_ctcss_tone()
-			if tone: self.debug("detect_ctcss: CTCSS tone %0.1f detected" %tone)
-			if tone == ctcss: return True
-			if timeout and time.time() >= timeout_time: break
-		return False
-
 	###################################
 	def delete_pidfile(self):
 		"""Delete pidfile after a daemon process has finished"""
@@ -661,6 +649,7 @@ class Phonepatch:
 		fd.write(str(os.getpid()) + "\n")
 		fd.close()
 		os.chown(self.pidfile, *self.get_asterisk_id())
+		self.pidfile_created = True
 
 	###################################
 	def read_pidfile(self):
@@ -683,56 +672,67 @@ class Phonepatch:
 	###################################
 	def check_ctcss(self, extension=None):
 		tone = self.radio.get_ctcss_tone()
-		if not tone: self.debug("check_ctcss: CTCSS not detected"); return
-		if extension == None: extensions = [s for s in self.configuration if s and s.isdigit()]
+		if not tone: return
+		if extension == None: extensions = self.phonepatch_extensions
 		else: extensions = [extension]
 		for section in extensions:
-			freq = self.get_ctcss_freq(self.getconf("ctcss", section))
+			freq = self.get_ctcss(self.getconf("ctcss_rx", section))
 			if freq and freq == tone:
-				self.debug("check_ctcss: CTCSS tone %0.1f found in phonepatch extension %s" %(tone, section))
 				return section
 		if extension == None:
 			self.debug("check_ctcss: CTCSS tone %0.1f not found in any phonepatch extension" %tone)
 		else: self.debug("check_ctcss: CTCSS tone %0.1f not found in phonepatch extension %d" %(tone, extension))
 
+	###################################
+	def set_ctcss_tx(self):
+		ctcss_tx_freq = self.get_ctcss(self.getconf("ctcss_tx"))
+		ctcss_tx_amplitude = self.get_ctcss(self.getconf("ctcss_tx_amplitude"))
+		try: ctcss_tx_freq = float(ctcss_tx_freq)
+		except: ctcss_tx_freq = None
+		if ctcss_tx_freq and ctcss_tx_amplitude: 
+			self.ctcss_tx = ctcss_tx_freq, ctcss_tx_amplitude
+			self.debug("loop_daemon: using ctcss_tx tone: %0.1f Hz, amplitude: %0.2f" %self.ctcss_tx)
+		else: self.ctcss_tx = None
+
 	###################################
 	def process_incall(self):
 		"""Waits for DTMF <answer_button> (with a timeout) and open the interface if received"""
 		incall = self.getconf("incall")
-		timeout_time = time.time() +self.getconf("incall_report_timeout")
+		calltimeout = time.time() +self.getconf("incall_report_timeout")
 		answer_button = self.getconf("incall_answer_button")
-		incall_ctcss = self.getconf("incall_ctcss")
-		if not incall_ctcss and self.getconf("incall_answer_mode") == "ctcss":
-			self.debug("process_incall: incall_answer_mode=ctcss but incall_ctcss not enabled")
-			return 
-		if incall_ctcss and not self.genctcss:
-			self.debug("process_incall: incall_ctcss enabled but destination has not CTCSS defined")
-			return 
+		mode = self.getconf("incall_answer_mode")
+		self.set_ctcss_tx()
+		ctcss_rx_freq = self.get_ctcss(self.getconf("ctcss_rx"))
+		if mode == "ctcss" and not ctcss_rx_freq:
+			self.debug("process_incall: incall_ctcss_mode set to 'ctcss' but parameter ctcss_rx not defined")
+			return
 		while 1:
 			# TODO: fullduplex
 			if self.play(True, True, self.getconf("incall_report_audio")) == None: 
 				self.debug("process_incall: play() ended abnormally")
-				return
-			if self.getconf("incall_answer_mode") == "open": 
+				return			
+			if not mode or mode == "open": 
 				self.debug("process_incall: answer mode set to open, opening channel")
 				return "answered"
-			elif self.getconf("incall_answer_mode") == "ctcss":
-				freq = self.genctcss[0]
-				self.debug("process_incall: waiting for CTCSS tone %0.1f" %freq)
-				try: ctcss_found = self.detect_ctcss(freq, self.getconf("incall_report_audio_wait"))
-				except: self.debug("process_incall: detect_ctcss ended"); return
-				if ctcss_found:
-					self.debug("process_incall: extension CTCSS tone %0.1f detected" %freq)
-					return "answered"
-			else:
-				if not answer_button: return "answered"
-				self.debug("process_incall: waiting for DTMF button: %s" %answer_button)
-				try: dtmf = self.detect_dtmf(answer_button, timeout=self.getconf("incall_report_audio_wait"), need_ctcss=incall_ctcss)
-				except IOError: self.debug("process_incall: detect_dtmf ended"); return
-				if dtmf:
-					self.debug("process_incall: DTMF answer button received")
-					return "answered"
-			if time.time() > timeout_time:
+			elif mode == "dtmf": self.debug("process_incall: waiting for DTMF button: %s" %answer_button)
+			elif mode == "ctcss": self.debug("process_incall: waiting for CTCSS tone %0.1f" %ctcss_rx_freq)
+			timeout = time.time() + self.getconf("incall_report_audio_wait")
+			while time.time() < timeout:
+				if not self.flush_asterisk(): return
+				buffer = self.radio.read_audio(self.buffer_size)
+				if mode == "dtmf":
+					keys = self.dtmf_decoder.decode_buffer(buffer)
+					for key in keys: self.debug("process_incall: DTMF button received: %s" %key)
+					if answer_button in keys:				
+						return "answered"
+				elif "ctcss" in modes:
+					self.radio.decode_ctcss(buffer)
+					tone = self.radio.get_ctcss_tone()
+					if tone == ctcss_rx_freq:
+						self.debug("process_incall: extension ctcss_rx tone %0.1f detected" %ctcss_rx_freq)
+						return "answered"
+			
+			if time.time() > calltimeout:
 				self.debug("process_incall: timeout reached: %d seconds" %self.getconf("incall_report_timeout"))
 				self.play(True, True, self.getconf("incall_report_timeout_audio"))
 				return
@@ -744,53 +744,54 @@ class Phonepatch:
 			return True
 		return False
 
+	###################################
+	def process_number(self, number):
+		# In CTCSS mode phonepatch extension is already set
+		if self.phonepatch_extension:
+			self.asterisk_extension = self.getconf("outcall_extension").replace("X", self.phonepatch_extension)
+			return number
+		# DTMF mode
+		if not self.getconf("outcall_dtmf_extension_mode"):
+			self.asterisk_extension = self.getconf("outcall_extension")
+			return number	
+		for phpext in self.phonepatch_extensions:				
+			mode = self.getconf("outcall_askfortone_mode", phpext)
+			dtmfid = self.getconf("outcall_dtmf_id", phpext).replace("X", phpext)
+			if mode == "dtmf" and dtmfid: 
+				if number[:len(dtmfid)] == dtmfid:
+					self.debug("process_number: phonepatch prefix match extension: %s" %phpext)
+					mode = self.getconf("outcall_askfortone_mode", phpext)
+					self.phonepatch_extension = phpext
+					number = number[len(dtmfid):]
+					self.asterisk_extension = self.getconf("outcall_extension").replace("X", phpext)
+					break
+		else: 
+			self.debug("process_number: outcall_dtmf_extension_mode enabled, but no outcall_dtmf_id matched the prefix number: %s" %number)
+			return
+		return number
+			
 	###################################
 	def make_call(self, number, reopen=True):
 		"""Use outgoing calls Asterisk facility to call <number>"""
-		number = str(number)		
-		extension = self.getconf("outcall_extension").replace("X", "")
-		if extension == None: extension = ""
-		else: extension = str(extension)
-		outcall_ctcss = self.getconf("outcall_ctcss")
-		
-		if not outcall_ctcss and self.getconf("outcall_extension_mode") == "ctcss":
-			self.debug("process_incall: outcall_extension_mode=ctcss but outcall_ctcss not enabled")
-			return 
-		if len(number) <= self.phonepatch_extension_length:
-			self.debug("make_call: destination number (%s) length is insufficient" %number)
-			return
-		if self.getconf("outcall_extension_mode") == "dtmf":
-			phonepatch_extension = number[:self.phonepatch_extension_length]
-			asterisk_extension = extension + phonepatch_extension
-			number = number[self.phonepatch_extension_length:]
-		else: asterisk_extension = phonepatch_extension = extension
-		
-		if self.getconf("outcall_extension_mode") == "ctcss":
-			phonepatch_extension = self.phonepatch_extension
-			if not phonepatch_extension: 
-				self.debug("make_call: phonepatch extension empty"); return
-			asterisk_extension += phonepatch_extension
-		if phonepatch_extension not in self.configuration:
-			self.debug("make_call: phonepatch extension not defined: %s" %phonepatch_extension)
-			return
-		self.phonepatch_extension = phonepatch_extension
-		phpname = self.getconf("phonepatch")
-		if phpname != self.phonepatch_php:
-			self.debug("make_call: phonepatch extension %s cannot use phonepatch %s (configured for %s)" %(phonepatch_extension, self.phonepatch_php, phpname))
+		number = self.process_number(str(number))
+		if not number: 
+			self.debug("make_call: process_number() not succesful")
 			return
 		if not self.getconf("outcall"):
-			self.debug("make_call: phonepatch extension %s not allowed to make outcalls" %phonepatch_extension)
+			self.debug("make_call: outcalls disabled for extension: %s" %self.phonepatch_extension)
 			return
-		self.debug("make_call: asterisk: %s - phonepatch: %s - destination: %s" %(asterisk_extension, phonepatch_extension, number))
+		
+		callerid = self.getconf("callerid")
+		if not callerid:
+			callerid = "%s <%s>" %(self.getconf("username"), self.phonepatch_extension)
+		self.debug("make_call: phonepatch: %s (%s) - destination: %s" %(self.phonepatch_extension, self.asterisk_extension, number))
 		channel = self.getconf("outcall_channel") + "/" + number
-		try: callerid = ("CallerID", "%s <%s>" %(self.getconf("name"), asterisk_extension))
-		except: 
-			self.debug("make_call: name parameter not defined for phonepatch extension %s" %phonepatch_extension)
-			return
+		
 		options = [("Channel", channel), ("MaxRetries", "0"), \
 			("RetryTime", "60"), ("Context", self.getconf("outcall_context")), \
-			("Extension", asterisk_extension), ("WaitTime", self.getconf("outcall_timeout")), \
-			("Priority", self.getconf("outcall_priority")), ("Account", OUTCALL_IDNAME), callerid]
+			("Extension", self.asterisk_extension), ("WaitTime", self.getconf("outcall_timeout")), \
+			("Priority", self.getconf("outcall_priority")), ("Account", OUTCALL_IDNAME), ("CallerID", callerid)]
+		
 		# Create a temporal file to write outgoing call options
 		tempfd, callpath = tempfile.mkstemp()
 
@@ -800,7 +801,7 @@ class Phonepatch:
 			self.debug("make_call: outcall - %s" %buffer)
 		os.close(tempfd)
 		
-		# Spool file must be owned by Asterisk
+		# Spool call file must be owned by Asterisk
 		uid, gid = self.get_asterisk_id()
 		os.chown(callpath, uid, gid)
 		
@@ -859,26 +860,26 @@ class Phonepatch:
 		self.debug("sleep_daemon: sleeping phonepatch until it receives a continue signal")
 		self.radio.close()
 		if self.radio_control: self.radio_control.close()
-		self.sleep_signal = False
 		
 		# Safe time gives some time extra to EAGI to finish
 		safe_time = 5.0
 		
-		# If there is an <call_limit>, the EAGI should end in that time, but
-		# for security set an alarm (with an extra <safe_time>) and wake up
+		# If there is an call_limit, the EAGI should end in that time, but
+		# for security set an alarm (with an extra safe_time) and wake up
 		if sleep_time: 
 			alarm_time = int(sleep_time + safe_time)
 			signal.alarm(alarm_time)
 			self.debug("sleep_daemon: timeout set to %d secs" %sleep_time)
 		
-		# Wait for <continue_signal>, which will be sent by the outcall EAGI phonepatch
-		self.stopped = True
-		while self.stopped:
+		# Wait for continue_signal, which will be sent by the outcall EAGI phonepatch
+		self.continue_signal = False
+		while not self.continue_signal:
 			time.sleep(0.1)
-		signal.alarm(0)		
+		signal.alarm(0)
 		if reopen:
 			self.debug("sleep_daemon: reopening phonepatch interface")
 			self.open_radio()
+		self.sleep_signal = False
 
 	#######################################
 	def check_daemon(self):
@@ -906,19 +907,13 @@ class Phonepatch:
 			syslog.openlog("phonepatch", syslog.LOG_PID, syslog.LOG_DAEMON)
 			self.modules_verbose = False
 		
-		extension = self.getconf("outcall_extension")
-		if not extension:
-			self.debug("init_daemon: parameter extension must be configured", 1)
 		pid = self.check_daemon()
-		if pid: self.debug("init_daemon: phonepatch daemon is already runnning with pid %d" %pid, 1, delete_pidfile=False)
-		try: 
-			xindex = extension.index("X")
-			self.phonepatch_extension_length = len(extension) - xindex
-			extension = extension[:xindex]
-		except: self.phonepatch_extension_length = 0
+		if pid: 
+			self.debug("init_daemon: phonepatch daemon is already runnning with pid %d" %pid, 1)
 			
 		# Init flag variables (pause and continue) and set signals
-		self.sleep_signal = self.stopped = False
+		self.sleep_signal = False
+		self.continue_signal = True
 		self.set_signals([signal.SIGHUP, signal.SIGUSR1, signal.SIGUSR2, \
 			signal.SIGALRM, signal.SIGTERM, signal.SIGINT])
 		self.create_pidfile()
@@ -953,18 +948,47 @@ class Phonepatch:
 	###################################
 	def loop_daemon(self):
 		# Wait for <asktone> button, record number and make a call when received <outcall_button>
-		outcall_ctcss = self.getconf("outcall_ctcss")
+		mode = self.getconf("outcall_askfortone_mode")
+		button = self.getconf("askfortone_button")
+		if not mode: self.debug("loop_daemon: outcall_askfortone_mode not defined"); return
 		while 1:
-			self.debug("loop_daemon: waiting for askfortone_button")
-			try: key = self.detect_dtmf(self.getconf("askfortone_button"), need_ctcss=outcall_ctcss)
-			except IOError: self.debug("loop_daemon: detect_dtmf ended"); break
-			if not key: break
-			
-			# Set CTCSS for output if enabled
-			if outcall_ctcss:
-				self.phonepatch_extension = self.check_ctcss()
-				self.set_ctcss(self.getconf("ctcss"), self.getconf("ctcss_amplitude"))
-				self.debug("loop_daemon: using CTCSS tone: %0.1f Hz, amplitude: %0.2f" %self.genctcss)
+			self.phonepatch_extension = None
+			if mode == "dtmf": self.debug("loop_daemon: waiting askfortone DTMF button: %s" %button)
+			self.debug("loop_daemon: CTCSS decoding enabled")
+			# CTCSS decoding always done (as it can be enabled inside an extension
+			# DTMF decoding only if asked globally
+			while 1:
+				if self.sleep_signal:
+					self.sleep_daemon(self.getconf("call_limit"))
+					continue
+
+				if not self.flush_asterisk(): return
+				buffer = self.radio.read_audio(self.buffer_size)
+				self.radio.decode_ctcss(buffer)
+				if mode == "dtmf":
+					keys = self.dtmf_decoder.decode_buffer(buffer)
+					for key in keys: self.debug("loop_daemon: DTMF button received: %s" %key)
+					if button in keys:				
+						break
+				extension = self.check_ctcss()
+				if not extension: continue
+				mode = self.getconf("outcall_askfortone_mode", extension)
+				if mode != "ctcss":
+					self.debug("loop_daemon: extension_askfortone_mode different from 'ctcss' in phonepatch: %s" %extension)
+					continue
+				number = self.getconf("outcall_ctcss_autocall", extension)
+				if number:
+					self.debug("loop_daemon: ctcss autocall: %s" %number)
+					self.phonepatch_extension = extension
+					if not self.make_call(number):
+						self.play(True, False, self.getconf("ring_timeout_audio"))
+					continue
+				elif extension:
+					self.debug("loop_daemon: ctcss_rx tone detected for extension: %s" %extension)
+					self.phonepatch_extension = extension
+					break
+
+			self.set_ctcss_tx()
 			
 			# AskForTone DTMF button received, now record the destination number
 			# TODO: Fullduplex
@@ -974,13 +998,9 @@ class Phonepatch:
 			dtmf_keys = []
 		
 			while 1:
-				if self.sleep_signal:
-					self.sleep_daemon(self.getconf("call_limit"))
-					break
-
 				now = time.time()
 				if now >= timeout_time:
-					self.debug("loop_daemon: wait for number timed out")
+					self.debug("loop_daemon: dial period number timed out")
 					self.play(True, False, self.getconf("tone_timeout_audio"))
 					break
 				buffer = self.radio.read_audio(self.buffer_size)
@@ -995,20 +1015,16 @@ class Phonepatch:
 					continue
 				if self.getconf("outcall_button") in dtmf_keys: 
 					dtmf_keys = dtmf_keys[:dtmf_keys.index(self.getconf("outcall_button"))]
-					if len(dtmf_keys) <= self.phonepatch_extension_length:
-						self.debug("loop_daemon: outcall_button received, but number length insufficient")
-						timeout_time = time.time() + self.getconf("tone_timeout")
-					else:
-						# We have a number (in a list) to call to, convert to string
-						number = "".join(dtmf_keys)						
-						noisy_button = self.getconf("dtmf_noisy_mode_button")
-						if noisy_button and noisy_button != "off":
-							number = self.process_noisy_number(number, noisy_button)
-						self.debug("loop_daemon: outcall_button received, making a call to %s" %number)
-						if not self.make_call(number):
-							self.play(True, False, self.getconf("ring_timeout_audio"))
-						dtmf_keys = []
-						break
+					# We have a number (in a list) to call to, convert to string
+					number = "".join(dtmf_keys)
+					noisy_button = self.getconf("dtmf_noisy_mode_button")
+					if noisy_button and noisy_button != "off":
+						number = self.process_noisy_number(number, noisy_button)
+					self.debug("loop_daemon: outcall_button received, making a call to %s" %number)
+					if not self.make_call(number):
+						self.play(True, False, self.getconf("ring_timeout_audio"))
+					dtmf_keys = []
+					break
 
 	###################################
 	def end_daemon(self):
@@ -1017,14 +1033,6 @@ class Phonepatch:
 		self.delete_pidfile()
 		self.debug("end_daemon: daemon ended")
 
-	###################################
-	def set_ctcss(self, ctcss_id, ctcss_amplitude):
-		freq = self.get_ctcss_freq(ctcss_id)
-		if freq:
-			self.genctcss = (freq, ctcss_amplitude)
-			self.debug("Using CTCSS tone: %0.1f Hz, amplitude: %0.2f" %self.genctcss)
-		else: self.genctcss = None
-
 	###################################
 	### MAIN FUNCTIONS: agicall(), incall(), outcall(), daemon()
 	###################################
@@ -1034,20 +1042,17 @@ class Phonepatch:
 		"""Phonepatch acting as AGI"""
 		self.set_state("agicall")
 		self.get_agi_keys(sys.stdin)
-		extensionagi = self.agi_keys["extension"]
-		if not extension:
-			self.debug("agicall: warning: phonepatch extension not given as first parameter, using the AGI key instead (%s) as Asterisk extension" %extension)
-			extension = extensionagi
+		#extension = self.agi_keys["extension"]
+		#if not extension:
+		#	self.debug("agicall: extension not found in AGI keys. Aborting"); return
 		if extension not in self.configuration:
 			self.debug("agicall: phonepatch extension not defined: %s" %extension, 1)
 		self.phonepatch_extension = extension
 		self.init_php(self.getconf("phonepatch"))
-		self.set_ctcss(self.getconf("ctcss"), self.getconf("ctcss_amplitude"))
 		if self.agi_keys.get("accountcode") == OUTCALL_IDNAME: 
 			self.outcall()
 		else: self.incall()
 		
-
 	###################################
 	def daemon(self, phonepatch, background=False, testcall=None):
 		"""Phonepatch acting as daemon.
@@ -1058,18 +1063,17 @@ class Phonepatch:
 		self.background = background
 		self.init_php(phonepatch)
 		if not self.getconf("outcall_daemon"):
+			if not background:
+				self.debug("daemon: outcall_daemon disabled, daemon not loaded for phonepatch: %s" %phonepatch)
 			return
 		if self.background:	
 			pid = daemonize.daemonize(return_child=True)
-			if pid: return
+			if pid: return pid
 		self.set_state("daemon")
 		self.init_daemon()
 		self.open_radio()
 		
 		if testcall != None:
-			if self.getconf("outcall_extension_mode") == "ctcss":
-				self.phonepatch_extension = testcall[:self.phonepatch_extension_length]
-				testcall = testcall[self.phonepatch_extension_length:]
 			if not self.make_call(testcall, reopen=False):
 				self.play(True, False, self.getconf("ring_timeout_audio"))
 			self.debug("daemon: test outcall ended")
@@ -1077,16 +1081,19 @@ class Phonepatch:
 			sys.exit(0)
 
 		self.loop_daemon()
-		return True
 
 	###################################
 	def outcall(self):
 		"""Phonepatch acting as outgoing caller"""
 		self.set_state("outcall")
+		if not self.getconf("outcall"):
+			self.debug("make_call: outcalls disabled for extension: %s" %self.phonepatch_extension)
+			return
+		self.set_ctcss_tx()
 		self.set_signals([signal.SIGHUP])
-		
 		# Send a stop_signal (SIGUSR1) to the active phonepatch
 		self.signal_daemon(signal.SIGUSR1)
+		self.send_signal = signal.SIGUSR2
 		self.open_radio()
 		self.open_interface(self.asterisk_fifo)
 		self.audio_loop()
@@ -1094,6 +1101,7 @@ class Phonepatch:
 		self.radio.close()
 
 		# The outcall has finished, tell the phonepatch daemon to continue
+		self.send_signal = None
 		self.signal_daemon(signal.SIGUSR2)
 		self.debug("daemon: outcall process ended")
 
@@ -1104,7 +1112,7 @@ class Phonepatch:
 		if not self.getconf("incall"):
 			self.debug("incall: phonepatch extension %s not allowed to receive incalls" %(self.phonepatch_extension))
 			return
-		self.destination =  self.getconf("name")
+		self.destination =  self.getconf("username")
 		self.set_signals([signal.SIGHUP])
 		
 		# Send a <stop_signal> to daemon process (if running)
@@ -1120,10 +1128,18 @@ class Phonepatch:
 		self.signal_daemon(signal.SIGUSR2)
 		self.debug("incall: process ended")
 
+####################################
+def get_phpconfigs(configuration):
+	phpconfigs = []
+	for section in configuration:
+		if type(section) == str and section.find("phonepatch") == 0:
+			phpconfigs.append(section)
+	return phpconfigs
+
 ###################################
 def main():
 	usage = """
-phonepatch [options] [extension]
+phonepatch [options] [extension_name]
 
 By default the phonepatch acts as daemon, opens radio interface 
 and spawn outgoing calls when required (radio DTMF controlled)"""
@@ -1141,32 +1157,30 @@ and spawn outgoing calls when required (radio DTMF controlled)"""
 	options, args = optpar.parse_args()
 	
 	config = templateparser.Parser(verbose = True)
-
 	config.read_template(default_template)
 	try: configuration = config.read_configuration(options.configuration_file)
 	except IOError, e: sys.stderr.write("configuration file not found: %s\n" %options.configuration_file); sys.exit(1)
-
+		
 	# Run daemon (default), incall or outcall mode
+	
 	execname = os.path.basename(sys.argv[0])
-	if execname == "phonepatch.agi": 
-		if len(args) < 1: extension = None
-		else: extension = args[0]
+	if len(args) > 0:
+		extname = args[0]
 		php = Phonepatch(configuration, verbose=options.verbose)
-		php.agicall(extension)
+		php.agicall(extname)
 	else: 
 		if options.phonepatch:
 			phonepatchs = [options.phonepatch]
 		else:
-			phonepatchs = [s for s in configuration if s and not s.isdigit() and s != GLOBAL_SECTION]
+			phonepatchs = get_phpconfigs(configuration)
 			if not phonepatchs: sys.stderr.write("no phonepatchs found in configuration\n"); sys.exit(1)
 			if not options.background: 
 				phonepatchs.sort()
 				phonepatchs = [phonepatchs[0]]
+				sys.stdout.write("using default phonepatch: %s\n" %phonepatchs[0])
 		for phpname in phonepatchs:
 			php = Phonepatch(configuration, verbose=options.verbose)
-			if not php.daemon(phpname, options.background, options.test_outcall):
-				sys.stdout.write("outcall daemon disabled for phonepatch: %s\n" %phpname)
-				sys.stdout.flush()
+			php.daemon(phpname, options.background, options.test_outcall)
 		
 	sys.exit(0)
 
